function MSignBootStrap(minimized) {
    this.minimized = minimized;
    this.scripts = {
        "full" : [
            "js/jquery-3.1.1.js",
            "js/base64-binary.js",
            "js/dialog-polyfill.js",
            "js/fullscreen-api-polyfill.js",
            "js/forge.min.js",
            "js/jsQR.js",
            "js/material.js",
            "js/msign-connection.js",
            "js/msign-device.js",
            "js/msign-device-mgr.js",
            "js/msign-display.js",
            "js/msign-dto.js",
            "js/msign-firmware.js",
            "js/msign-regions.js",
            "js/msign-reports.js",
            "js/msign-usb.js",
            "js/msign-version.js",
            "js/msign-webapp.js",
            "js/msign-webcam.js",
            "js/msign-easyscript.js"
        ],
        "minimized" : [
            "js/msign.min.js",
            "js/forge.min.js"
        ]
    };

    this.styles = {
        "full": [
            "css/msign.css",
            "css/dialog-polyfill.css",
            "css/material.css",
            "css/material-icons.css"
        ],
        "minimized": [
            "css/msign.min.css"
        ]
    };

    this.currentStep = 0;
    this.stepCount = 1;
    this.checkedKeys = false;
    this.progressBar = false;
    this.step = false;
    this.lastStep = "";
    this.isDone = false;
    this.maxTime = 60 * 1000;
}

MSignBootStrap.prototype.contentLoaded = function() {
    this.progressBar = document.getElementById("splash_screen_progress_filled");
    this.step = document.getElementById("splash_screen_step");
    this.hideContent();
    this.load();

    var t = this;
    setTimeout(function() {
        if(!t.isDone) {
            //It's not going to finish, probably.
            location.reload();
        }
    }, this.maxTime);
};

MSignBootStrap.prototype.hideContent = function() {
    var cards = document.getElementsByClassName("mdl-layout");
    for(var i = 0; i < cards.length; i++) {
        cards[i].style.display = "none";
    }

    var hiddenStuff = document.getElementsByClassName("scriptel_hidden");
    for(var i = 0; i < hiddenStuff.length; i++) {
        hiddenStuff[i].style.display = "none";
    }
};

MSignBootStrap.prototype.done = function() {
    this.isDone = true;
    var splash = document.getElementById("splash_screen");
    splash.parentNode.removeChild(splash);
    var screens = document.getElementById("screens");
    screens.style.display = "block";
    var loginScreen = document.getElementById("login_screen");
    loginScreen.style.display = "";
    loginScreen.className = loginScreen.className.replace("scriptel_hidden", "");
    msignApp = new MSignWebApplication();
    msignApp.initialize();
};

MSignBootStrap.prototype.load = function() {
    var t = this;
    var styles = this.styles[this.minimized ? "minimized" : "full"];
    styles.forEach(function(source) {
        var link = document.createElement("link");
        link.addEventListener("load", function() { t.assetLoaded(source); }, false);
        link.setAttribute("rel", "stylesheet");
        link.setAttribute("type", "text/css");
        link.setAttribute("href", source);
        document.head.appendChild(link);
        t.stepCount++;
    });

    var scripts = this.scripts[this.minimized ? "minimized" : "full"];
    scripts.forEach(function(source) {
        var script = document.createElement("script");
        script.addEventListener("load", function() { t.assetLoaded(source); }, false);
        script.setAttribute("type", "application/javascript");
        script.setAttribute("src", source);
        script.async = false;
        document.head.appendChild(script);
        t.stepCount++;
    });
};

MSignBootStrap.prototype.setProgress = function(desc) {
    this.progressBar.style.width = ((this.currentStep / this.stepCount) * 100) + "%";
    if(desc !== this.lastStep) {
        while(this.step.firstChild) {
            this.step.removeChild(this.step.firstChild);
        }
        this.step.appendChild(document.createTextNode(desc));
        this.lastStep = desc;
    }

    var diff = this.stepCount - this.currentStep;
    if(!this.checkedKeys && diff === 1) {
        this.generateKeys();
    } else if(diff === 0) {
        this.done();
    }
};

MSignBootStrap.prototype.assetLoaded = function(asset) {
    this.currentStep++;
    this.setProgress("Loading Assets");
};

MSignBootStrap.prototype.generateKeys = function() {
    this.checkedKeys = true;
    if(localStorage.getItem("msign.key.public") === null || localStorage.getItem("msign.key.private") === null) {
        this.setProgress("Generating Key Pair");
        var pki = forge.pki;
        var rsa = pki.rsa;
        var t = this;
        rsa.generateKeyPair({bits: 2048, workers: 2}, function(err, keypair) {
            if(!err) {
                localStorage.setItem("msign.key.public", pki.publicKeyToPem(keypair.publicKey));
                localStorage.setItem("msign.key.private", pki.privateKeyToPem(keypair.privateKey));
                t.currentStep++;
                t.setProgress("Done!");
            }
        });
    } else {
        this.currentStep++;
        this.setProgress("Done!");
    }
}

MSignBootStrap.prototype.initialize = function() {
    var t = this;
    document.addEventListener("DOMContentLoaded", function() { t.contentLoaded(); }, false);
};

//Start up the bootstrapper ASAP.
new MSignBootStrap(true).initialize();